@@ -59,22 +59,24 @@ class Attention(nn.Module):
 class SpatialGatingUnit(nn.Module):
     def __init__(self, dim, dim_seq, attn_dim = None, causal = False):
         super().__init__()
+        dim_out = dim // 2
         self.causal = causal
 
-        self.norm = nn.LayerNorm(dim)
+        self.norm = nn.LayerNorm(dim_out)
         self.proj = nn.Conv1d(dim_seq, dim_seq, 1)
-        self.attn = Attention(dim * 2, dim, attn_dim, causal) if exists(attn_dim) else None
+        self.attn = Attention(dim, dim_out, attn_dim, causal) if exists(attn_dim) else None
         nn.init.zeros_(self.proj.weight)
         nn.init.constant_(self.proj.bias, 1.)
 
     def forward(self, x):
-        device = x.device
+        device, n = x.device, x.shape[1]
 
         res, gate = x.chunk(2, dim = -1)
         gate = self.norm(gate)
 
         weight, bias = self.proj.weight, self.proj.bias
         if self.causal:
+            weight, bias = weight[:n, :n], bias[:n]
             mask = torch.ones(weight.shape[:2], device = device).triu_(1).bool()
             weight = weight.masked_fill(mask[..., None], 0.)
 
@@ -108,10 +110,10 @@ class gMLP(nn.Module):
 
         self.layers = nn.ModuleList([Residual(nn.Sequential(
             nn.LayerNorm(dim),
-            nn.Linear(dim, dim_ff * 2),
+            nn.Linear(dim, dim_ff),
             nn.GELU(),
             SpatialGatingUnit(dim_ff, seq_len, attn_dim, causal),
-            nn.Linear(dim_ff, dim)
+            nn.Linear(dim_ff // 2, dim)
         )) for i in range(depth)])
 
         self.to_logits = nn.Sequential(
@@ -153,10 +155,10 @@ class gMLPVision(nn.Module):
 
         self.layers = nn.ModuleList([Residual(nn.Sequential(
             nn.LayerNorm(dim),
-            nn.Linear(dim, dim_ff * 2),
+            nn.Linear(dim, dim_ff),
             nn.GELU(),
             SpatialGatingUnit(dim_ff, num_patches, attn_dim),
-            nn.Linear(dim_ff, dim)
+            nn.Linear(dim_ff // 2, dim)
         )) for i in range(depth)])
 
         self.to_logits = nn.Sequential(
